# -*- coding: utf-8 -*-
from __future__ import absolute_import, division, with_statement
import contextlib
import socket
import errno
import random
from port_for import ephemeral, utils
from ._ranges import UNASSIGNED_RANGES
from .exceptions import PortForException

SYSTEM_PORT_RANGE = (0, 1024)

def select_random(ports=None, exclude_ports=None):
    """
    Returns random unused port number.
    """
    if ports is None:
        ports = available_good_ports()

    if exclude_ports is None:
        exclude_ports = set()

    ports.difference_update(set(exclude_ports))

    for port in random.sample(ports, min(len(ports), 100)):
        if not port_is_used(port):
            return port
    raise PortForException("Can't select a port")

def is_available(port):
    """
    Returns if port is good to choose.
    """
    return port in available_ports() and not port_is_used(port)

def available_ports(low=1024, high=65535, exclude_ranges=None):
    """
    Returns a set of possible ports (excluding system,
    ephemeral and well-known ports).

    Pass ``high`` and/or ``low`` to limit the port range.
    """
    if exclude_ranges is None:
        exclude_ranges = []
    available = utils.ranges_to_set(UNASSIGNED_RANGES)
    exclude = utils.ranges_to_set(
        ephemeral.port_ranges() + exclude_ranges +
        [
            SYSTEM_PORT_RANGE,
            (SYSTEM_PORT_RANGE[1], low),
            (high, 65536)
        ]
    )
    return available.difference(exclude)

def good_port_ranges(ports=None, min_range_len=20, border=3):
    """
    Returns a list of 'good' port ranges.
    Such ranges are large and don't contain ephemeral or well-known ports.
    Ranges borders are also excluded.
    """
    min_range_len += border*2
    if ports is None:
        ports = available_ports()
    ranges = utils.to_ranges(list(ports))
    lenghts = sorted([(r[1]-r[0], r) for r in ranges], reverse=True)
    long_ranges = [l[1] for l in lenghts if l[0] >= min_range_len]
    without_borders = [(low+border, high-border) for low, high in long_ranges]
    return without_borders

def available_good_ports(min_range_len=20, border=3):
    return utils.ranges_to_set(
        good_port_ranges(min_range_len=min_range_len, border=border)
    )


def port_is_used(port, host='127.0.0.1'):
    """
    Returns if port is used. Port is considered used if the current process
    can't bind to it or the port doesn't refuse connections.
    """
    unused = _can_bind(port, host) and _refuses_connection(port, host)
    return not unused


def _can_bind(port, host):
    sock = socket.socket()
    with contextlib.closing(sock):
        try:
            sock.bind((host, port))
        except socket.error:
            return False
    return True

def _refuses_connection(port, host):
    sock = socket.socket()
    with contextlib.closing(sock):
        sock.settimeout(1)
        err = sock.connect_ex((host, port))
        return err == errno.ECONNREFUSED



# -*- coding: utf-8 -*-
from __future__ import absolute_import, with_statement
import os
try:
    from ConfigParser import ConfigParser, DEFAULTSECT
except ImportError: # python3
    from configparser import ConfigParser, DEFAULTSECT

from .api import select_random
from .exceptions import PortForException

DEFAULT_CONFIG_PATH = '/etc/port-for.conf'

class PortStore(object):
    def __init__(self, config_filename=DEFAULT_CONFIG_PATH):
        self._config = config_filename

    def bind_port(self, app, port=None):
        if '=' in app or ':' in app:
            raise Exception('invalid app name: "%s"' % app)

        if port is not None:
            port = str(port)

        parser = self._get_parser()

        # this app already use some port; return it
        if parser.has_option(DEFAULTSECT, app):
            actual_port = parser.get(DEFAULTSECT, app)
            if port is not None and port != actual_port:
                msg = "Can't bind to port %s: %s is already associated with port %s" % (port, app, actual_port)
                raise PortForException(msg)
            return int(actual_port)

        # port is already used by an another app
        app_by_port = dict((v, k) for k, v in parser.items(DEFAULTSECT))
        bound_port_numbers = map(int, app_by_port.keys())

        if port is None:
            port = str(select_random(exclude_ports=bound_port_numbers))

        if port in app_by_port:
            binding_app = app_by_port[port]
            if binding_app != app:
                raise PortForException('Port %s is already used by %s!' % (port, binding_app))

        # new app & new port
        parser.set(DEFAULTSECT, app, port)
        self._save(parser)

        return int(port)

    def unbind_port(self, app):
        parser = self._get_parser()
        parser.remove_option(DEFAULTSECT, app)
        self._save(parser)

    def bound_ports(self):
        return [(app, int(port)) for app, port in self._get_parser().items(DEFAULTSECT)]

    def _ensure_config_exists(self):
        if not os.path.exists(self._config):
            with open(self._config, 'wb'):
                pass

    def _get_parser(self):
        self._ensure_config_exists()
        parser = ConfigParser()
        parser.read(self._config)
        return parser

    def _save(self, parser):
        with open(self._config, 'wt') as f:
            parser.write(f)




# -*- coding: utf-8 -*-
from __future__ import absolute_import
__version__ = '0.3'

from .api import (available_good_ports, available_ports, is_available,
                  good_port_ranges, port_is_used, select_random,
                  UNASSIGNED_RANGES)
from .store import PortStore
from .exceptions import PortForException



#!/usr/bin/env python
from distutils.core import setup

for cmd in ('egg_info', 'develop'):
    import sys
    if cmd in sys.argv:
        from setuptools import setup

version='0.3'

setup(
    name='port-for',
    version=version,
    author='Mikhail Korobov',
    author_email='kmike84@gmail.com',

    packages=['port_for'],
    scripts=['scripts/port-for'],

    url='https://github.com/kmike/port-for/',
    license = 'MIT license',
    description = """Utility that helps with local TCP ports managment. It can find an unused TCP localhost port and remember the association.""",

    long_description = open('README.rst').read(),

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',

        'Operating System :: POSIX',
        'Topic :: System :: Installation/Setup',
        'Topic :: System :: Systems Administration',
        'Topic :: Internet :: WWW/HTTP :: Site Management',
    ],
)
